/*
* igDragDrop.js
* Version 9.1.20091.1015
* Copyright(c) 2001-2009 Infragistics, Inc. All Rights Reserved.
*/


Type.registerNamespace("Infragistics.Web.UI");
var $IG = Infragistics.Web.UI;

$IG.DragDropEffects = function()
{
	///<summary>
	/// The enumerator which is used as parameter for get_dragDropEffect method of DragDropManager and
	/// set_dragDropMode method of $IG.DragDropBehavior.
	/// The value of None means no effect.
	/// The value of Move means move effect regardless of Ctrl key.
	/// The value of Copy means copy effect regardless of Ctrl key.
	/// The value of Default means copy effect if Ctrl key is pressed and move is Ctrl key is not pressed.
	///</summary>
}
$IG.DragDropEffects.prototype = 
{
	None:0,
	Move:1,
	Copy:2,
	Default:3
};
$IG.DragDropEffects.registerEnum("Infragistics.Web.UI.DragDropEffects");

$IG.DragDropAction = function()
{
	///<summary>
	/// The enumerator which is used as parameter for set_defaultDropAction method of $IG.DragDropBehavior.
	/// The value of None means that application should implement drop action.
	/// Otherwise, action will depend on get_dragDropEffect() of DragDropManager.
	/// If dragDropEffect is Copy and value is set to Append, then
	/// the copy/clone of source element will be appended to target.
	/// If dragDropEffect is Copy and value is set to Insert, then
	/// the copy/clone of source element will be inserted in the target as first child.
	/// If dragDropEffect is Move and value is set to Append, then source
	/// element will be removed from it original location and appended to target.
	/// If dragDropEffect is Move and value is set to Insert, then source
	/// element will be removed from it original location and inserted into target as first child.
	/// So, source element can be dragged and dropped on another target.
	///</summary>
}
$IG.DragDropAction.prototype = 
{
	None:0,
	Append:1,
	Insert:2
};
$IG.DragDropAction.registerEnum("Infragistics.Web.UI.DragDropAction");

$IG._DragDropManager = function()
{
	this._dragging = false;
	this._currentTarget = null;
	this._currentSource = null;
	this._zIndex = 99999;
	this._ctrl = false;
	this._shift = false;
	this._alt = false;
	this._targetElements = [];
	this._dragDropEffect = $IG.DragDropEffects.Default;
	this._supportsElemFromPoint = document.elementFromPoint != null;
	
	this._mouseUpDelegate = Function.createDelegate(this, this._onMouseUp);
	this._mouseOutDelegate = Function.createDelegate(this, this._onMouseOut);
	this._mouseMoveShellDelegate = Function.createDelegate(this, this._onMouseMoveShell);
	if (window.navigator.userAgent.indexOf("MSIE 8.0") >= 0) //$util is not available at this point
		this._mouseMoveEFPDelegate = Function.createDelegate(this, this._onMouseMoveElemFromPoint);
	this._mouseMoveDelegate = Function.createDelegate(this, this._onMouseMove);
	this._selectDelegate = Function.createDelegate(this, this._onSelectStart);
	this._keyDelegate = Function.createDelegate(this, this._onKey);
}
$IG._DragDropManager.prototype =
{
	
	get_isDragging: function()
	{
		/// <summary>Checks state of Drag and Drop.</summary>
		/// <returns type="Boolean">True: Drag and Drop operation is currently occurring.</returns>
		return this._dragging;
	},
	isCtrlKey: function()
	{
		/// <summary>Checks if Ctrl key is pressed.</summary>
		/// <returns type="Boolean">True: Ctrl key is pressed.</returns>
		return this._ctrl;
	},
	isShiftKey: function()
	{
		/// <summary>Checks if Shift key is pressed.</summary>
		/// <returns type="Boolean">True: Shift key is pressed.</returns>
		return this._shift;
	},
	isAltKey: function()
	{
		/// <summary>Checks if Alt key is pressed.</summary>
		/// <returns type="Boolean">True: Alt key is pressed.</returns>
		return this._alt;
	},
	get_source: function()
	{
		///<summary>
		/// If a Drag operation is occurring, it will return an Object that contains
		/// the specified source html "element" member variable and if specified,
		/// then the "object" member variable. Otherwise it will return null.
		///</summary>
		/// <returns type="Object" mayBeNull="true">Reference to object with "element" and "object" member variables.</returns>
		return this._currentSource;
	},

	get_sourceElement: function()
	{
		///<summary>
		/// If a Drag operation is occurring, it will return the specific HTML element that the drag 
		/// operation started on. Note, this can be different than the element returned by the 
		/// get_source() method, as it may be a child element of the source element. Otherwise it will 
		/// return null. 
		///</summary>
		/// <returns type="Object" mayBeNull="true">Reference to html element.</returns>
		if (this._sourceBehavior)
			return this._sourceBehavior.__startElem;
		return null;
	},

	get_target: function(behavior)
	{
		///<summary>
		/// If a Drag operation is occurring and the mouse is currently over a Target element, 
		/// it will return an object that contains the specified Target Element and, if specified,
		/// the Target Object.  Otherwise it will return null.
		///</summary>
		///<param name="behavior"  optional="true" mayBeNull="true">
		/// The behavior for the target you want returned. If no behavior is specified, the first target will be returned. 
		///</param>
		if (this._currentTarget)
		{
			if (behavior)
			{
				var targets = this._currentTarget.target.targets;
				for (var i = 0; i < targets.length; i++)
				{
					var target = targets[i];
					if (target.behavior == behavior)
						return target;
				}
			}
			else
			{
				return this._currentTarget.target.targets[0];
			}
		}
		return null;
	},

	get_targetElement: function()
	{
		///<summary>
		/// If a Drag operation is occurring and the mouse is currently over a Target element, 
		/// it will return the specific HTML element that the mouse is currently over. Note, this 
		/// can be different than the element returned by the get_target() method, as it may be a 
		/// child element of the target element. Otherwise it will return null. 
		///</summary>
		/// <returns type="Object" mayBeNull="true">Reference to html element under mouse.</returns>
		if (this._currentTarget)
			return this._currentTarget.elemAtPoint;
		return null;
	},

	get_dragDropEffect: function()
	{
		///<summary>
		/// Returns the current DragDropEffect of the current Drag operation, if one is occurring.
		///</summary>
		///<summary>
		/// That method can be used while processing the DragEnd event, when DragDropBehavior.set_defaultDropAction was set.
		///</summary>
		/// <returns type="Infragistics.Web.UI.DragDropEffects">Returns current drag effect.</returns>
		if (this._sourceBehavior && this._sourceBehavior._dragDropMode != $IG.DragDropEffects.Default)
			return this._sourceBehavior._dragDropMode;
		return this._dragDropEffect;
	},

	_set_dragDropEffect: function(effect)
	{
		this._dragDropEffect = effect;
		this._updateCursor(effect);
	},

	get_dataObject: function()
	{
		///<summary>
		/// Gets the data of the current Drag operation. Note, if a drag operation is not 
		/// occurring it will return null. 
		///</summary>
		/// <returns type="Object" mayBeNull="true">Reference to object which was set by set_dataObject or null.</returns>
		return this._dataObject;
	},

	set_dataObject: function(data)
	{
		///<summary>
		/// Sets the data of the current Drag operation.
		///</summary>
		///<param name="data" mayBeNull="true">Object associated with drag operation.</param>
		this._dataObject = data;
	},

	get_dataText: function()
	{
		///<summary>
		/// Gets the data text of the current Drag operation. Note, if a drag operation is 
		/// not occurring it will return null. 
		///</summary>
		/// <returns type="Object" mayBeNull="true">Reference to string set by set_dataText.</returns>
		return this._dataText;
	},
	set_dataText: function(text)
	{
		///<summary>
		/// Gets the data text of the current Drag operation. Note, if a drag operation is 
		/// not occurring it will return null. 
		///</summary>
		///<param name="text" mayBeNull="true">Text associated with drag operation.</param>
		this._dataText = text;
	},

	get_dragMarkupElement: function()
	{
		///<summary>
		/// If a Drag operation is currently occurring, it will return the actual HTML element 
		/// that is being used to drag around. This information is useful if you want to know the 
		/// top and left coordinates of the dragging element, instead of the x and y coordinates 
		/// of mouse, as they may be different, depending on the browser. Otherwise it will return null.
		///</summary>
		/// <returns type="Object" mayBeNull="true">Reference to drag markup element or null.</returns>
		return this._draggingMarkup;
	},
	get_defaultActionDropElement: function()
	{
		///<summary>
		/// That method can be used while processing the DragEnd event, when DragDropBehavior.set_defaultDropAction was set.
		///</summary>
		/// <returns type="Object" mayBeNull="true">Reference to dropped element or null.</returns>
		return this._defDropElem;
	},
	

	
	endDrag: function(cancel)
	{
		///<summary>
		/// This method will end the current drag operation, if one is currently occurring. 
		///</summary>
		///<param name="cancel" type="Boolean" optional="true" mayBeNull="true">
		/// Determines if the Drop event should fire if the mouse is currently over a target element.
		///</param>
		var beh = this._sourceBehavior;
		if (!this._dragging || !beh)
			return;
		if (this._currentTarget && !cancel)
		{
			if (this.__fireTargetEvent("Drop"))
				this._defaultDrop(beh);
		}
		else
			beh._events._fireEvent("DragCancel");
		beh._events._fireEvent("DragEnd");
		this._dragging = this._shift = this._alt = this._ctrl = false;
		if (this._draggingMarkup)
			document.body.removeChild(this._draggingMarkup);
		this.__setCursor('', true);
		if (this._dragShell)
			this._dragShell.style.display = "none";
		this._dragDropEffect = $IG.DragDropEffects.Default;
		this._dataObject = this._dataText = this._draggingMarkup = this._dragShell = this._currentTarget = this._currentSource = this._sourceBehavior = this._defDropElem = null;
		if (this._dragTime > 0)
		{
			
			$removeHandler(document, 'selectstart', this._selectDelegate);
			$removeHandler(document, 'keydown', this._keyDelegate);
			$removeHandler(document, 'keyup', this._keyDelegate);
			$removeHandler(document, 'mouseout', this._mouseOutDelegate);
			$removeHandler(document, 'mouseup', this._mouseUpDelegate);
			if (!this._supportsElemFromPoint)
			{
				this.__handleWindowedElems(false);
				$removeHandler(document, 'mousemove', this._mouseMoveDelegate);
			}
			else if (this._docMove)
			{
				$removeHandler(document, 'mousemove', this._mouseMoveShellDelegate);
				$removeHandler(document, 'scroll', this._mouseUpDelegate);
			}
		}
		this._dragTime = 0;
	},

	elementFromPoint: function(x, y)
	{
		if (!this._supportsElemFromPoint)
			return null;
		var efp = document.elementFromPoint(x, y);
		
		if (efp == null)
			efp = this._elementFromPoint;
		return efp;
	},

	dispose: function()
	{
		///<summary>
		/// Removes event handlers and clears objects.
		///</summary>
		this.endDrag(true);
		var shell = this._dragShell;
		if (!shell)
			return;
		if (this._shellLsnr)
		{
			var doc = shell.contentWindow.document;
			$util.removeHandler(doc, 'mousemove', this._mouseMoveShellDelegate);
			if ($util.IsIE8)
				$util.removeHandler(doc, 'mousemove', this._mouseMoveEFPDelegate);
			$util.removeHandler(doc, 'mouseup', this._mouseUpDelegate);
		}
		if (this._shellLoadDelegate)
			$removeHandler(shell, 'load', this._shellLoadDelegate);
		document.body.removeChild(shell);
		this._dragShell = null;
	},
	

	
	_fixCtrl: function(e, cursor)
	{
		if (!e)
			return;
		this._shift = e.shiftKey;
		this._alt = e.altKey;
		var ctrl = e.ctrlKey;
		if (ctrl == this._ctrl)
			return;
		this._ctrl = ctrl;
		if (cursor)
			this._validateCursor(e);
	},
	_onSelectStart: function(e)
	{
		$util.cancelEvent(e);
	},
	_onMouseOut: function(e)
	{
		
		if ((new Date()).getTime() < this._dragTime + 200)
			return;
		
		var elem = this._supportsElemFromPoint ? null : e.target;
		if (!elem || elem.nodeName == 'HTML')
			this.endDrag();
	},
	_onMouseUp: function(e)
	{
		this._fixCtrl(e);
		this.endDrag();
	},
	_onKey: function(e)
	{
		if (e.keyCode == Sys.UI.Key.esc)
			this.endDrag(true);
		this._fixCtrl(e, 1);
	},
	
	
	
	
	
	
	
	
	
	_onMouseMoveShell: function(e)
	{
		var elem = this._draggingMarkup, scroll = ($util.IsMac && $util.IsSafari) ? document.body : document.documentElement;
		if (!elem || !this._dragging)
			return;
		elem.style.top = (elem._originalTop - (elem._originalY - e.clientY)) + 'px';
		elem.style.left = (elem._originalLeft - (elem._originalX - e.clientX)) + 'px';
		var x = scroll.scrollLeft, y = scroll.scrollTop;
		if (this._docMove)
		{
			x = -x;
			y = -y;
		}
		var targetObj = this._getTargetFromPoint(e.clientX - x, e.clientY - y);
		this._validateTarget(targetObj, e);
	},
	
	_elementFromPoint: null,
	_deferredEFPDelegate: null,
	_efpX: -1,
	_efpY: -1,
	_onMouseMoveElemFromPoint: function(e)
	{
		if (!this._deferredEFPDelegate)
			this._deferredEFPDelegate = Function.createDelegate(this, this._checkElementFromPoint);
		this._efpX = e.clientX;
		this._efpY = e.clientY;
		if (this._draggingMarkup)
			this._draggingMarkup.style.display = "none";
		if (this._dragShell)
			this._dragShell.style.display = "none";
		window.setTimeout(this._deferredEFPDelegate);
	},
	_checkElementFromPoint: function()
	{
		this._elementFromPoint = document.elementFromPoint(this._efpX, this._efpY);
		if (this._draggingMarkup)
			this._draggingMarkup.style.display = "";
		if (this._dragShell)
			this._dragShell.style.display = "";
	},
	
	_onMouseMove: function(e)
	{
		var elem = e.target;
		if (!elem || !this._dragging)
			return;
		if (this._currentElement != elem)
		{
			this._previousElement = this._currentElement;
			this._currentElement = elem;
		}
		else
			this._previousElem = null;
		var targetObj = this._validateElem(elem);
		elem = this._draggingMarkup;
		if (!elem)
			return;
		elem.style.top = (e.clientY + document.documentElement.scrollTop + 1) + 'px';
		elem.style.left = (e.clientX + document.documentElement.scrollLeft + 1) + 'px';
		this._validateTarget(targetObj, e);
	},
	_onLoadFrame: function()
	{
		var win = this._dragShell;
		if (!win || this._cursorElem)
			return;
		win = win.contentWindow;
		var doc = win.document;
		
		var body = doc.body, div = doc.createElement('DIV');
		var style = div.style;
		body.appendChild(div);
		body.leftMargin = body.rightMargin = body.topMargin = body.bottomMargin = '0';
		body.style.height = style.height = style.width = '100%';
		style.position = 'absolute';
		style.top = style.left = '0px';
		this._cursorElem = div;
		
		if (!this._docMove)
		{
			this._shellLsnr = true;
			$util.addHandler(doc, 'mousemove', this._mouseMoveShellDelegate);
			if ($util.IsIE8)
				$util.addHandler(doc, 'mousemove', this._mouseMoveEFPDelegate);
			$util.addHandler(doc, 'mouseup', this._mouseUpDelegate);
		}
	},
	

	
	_startDrag: function(dragBehavior, source, e)
	{
		this.endDrag();
		if (!dragBehavior)
			return;
		var x = e.clientX, y = e.clientY;

		this._currentSource = source;
		this._sourceBehavior = dragBehavior;
		if (dragBehavior._events._fireEvent("DragStart", { x: x, y: y }))
		{
			this._dragging = true;
			this._docMove = $util.IsOpera || $util.IsMac;

			if (this._supportsElemFromPoint)
				this._showShell();
			this.__setupVisibleDragMakup(x, y, dragBehavior._underMouseElem ? dragBehavior.__startElem : source.element);
			this._dragTime = (new Date()).getTime();
			
			$addHandler(document, 'selectstart', this._selectDelegate);
			$addHandler(document, 'keydown', this._keyDelegate);
			$addHandler(document, 'keyup', this._keyDelegate);
			$addHandler(document, 'mouseout', this._mouseOutDelegate);
			$addHandler(document, 'mouseup', this._mouseUpDelegate);
			if (!this._supportsElemFromPoint)
			{
				this.__handleWindowedElems(true);
				$addHandler(document, 'mousemove', this._mouseMoveDelegate);
			}
			else if (this._docMove)
			{
				$addHandler(document, 'mousemove', this._mouseMoveShellDelegate);
				$addHandler(document, 'scroll', this._mouseUpDelegate);
			}
			this._fixCtrl(e);
		}
		else
			this.endDrag();
	},

	_validateTarget: function(targetObj, e)
	{
		if (targetObj)
		{
			var targets = targetObj.target.targets;
			var fireDragEnter = false;
			for (var i = 0; i < targets.length; i++)
			{
				var target = targets[i];
				var elem = target._includeChildren ? targetObj.elemAtPoint : targetObj.element;
				if (!this._currentTarget)
				{
					fireDragEnter = true;
					break;
				}
				var currentTargets = this._currentTarget.target.targets;
				for (var j = 0; j < currentTargets.length; j++)
				{
					var currentTarget = currentTargets[j];
					var currentTargetElem = currentTarget._includeChildren ? this._currentTarget.elemAtPoint : this._currentTarget.element;
					if (elem != currentTargetElem)
					{
						this.__fireTargetEvent("DragLeave");
						fireDragEnter = true;
					}
				}
			}
			if (fireDragEnter)
			{
				this._currentTarget = targetObj;
				if (!this.__fireTargetEvent("DragEnter"))
				{
					this._set_dragDropEffect($IG.DragDropEffects.None);
					this._currentTarget = null;
					return;
				}
			}
			this.__fireTargetEvent("DragMove", { x: e.clientX, y: e.clientY });
			this._validateCursor(e);
		}
		else
		{
			if (this._currentTarget)
				this.__fireTargetEvent("DragLeave");
			this._set_dragDropEffect($IG.DragDropEffects.None);
			this._currentTarget = null;
		}
	},

	_validateCursor: function(e)
	{
		var src = this._sourceBehavior;
		if (!src)
			return;
		var mode = src._dragDropMode;
		if (mode == $IG.DragDropEffects.Default)
			this._set_dragDropEffect(e.ctrlKey ? $IG.DragDropEffects.Copy : $IG.DragDropEffects.Move);
		else
			this._updateCursor(mode)
	},

	_updateCursor: function(effect)
	{
		if (!this._dragging)
			return;
		if (effect == $IG.DragDropEffects.None)
			this.__setCursor(this._sourceBehavior._noneCursor);
		else if (effect == $IG.DragDropEffects.Move)
			this.__setCursor(this._sourceBehavior._moveCursor);
		else if (effect == $IG.DragDropEffects.Copy)
			this.__setCursor(this._sourceBehavior._copyCursor);
	},

	_getTargetFromPoint: function(x, y)
	{
		this._draggingMarkup.style.display = this._dragShell.style.display = "none";
		var elemAtPoint = this.elementFromPoint(x, y);
		this._draggingMarkup.style.display = this._dragShell.style.display = "";
		return this._validateElem(elemAtPoint);
	},

	_validateElem: function(elemAtPoint)
	{
		if (!elemAtPoint)
			return null;
		var targets = this._targetElements;
		var i = targets ? targets.length : 0;
		while (i-- > 0)
		{
			var target = targets[i];
			var elem = target.element;
			if (elem != null && (elem == elemAtPoint || ((elem.contains && elem.contains(elemAtPoint)) || (!elem.contains && $util.isChild(elem, elemAtPoint)))))
				return { target: target, element: elem, elemAtPoint: elemAtPoint };
		}
		return null;
	},

	_registerTarget: function(behavior, target)
	{
		var targetElementIndex = -1;
		var targets = this._targetElements;
		for (var i = 0; i < targets.length; i++)
		{
			if (targets[i].element == target.element)
			{
				targetElementIndex = i;
				break;
			}
		}
		if (targetElementIndex == -1)
		{
			targetElementIndex = targets.length;
			targets.push({ element: target.element, targets: [] });
		}
		target.behavior = behavior;
		targets[targetElementIndex].targets.push(target);
	},

	_unRegisterTarget: function(behavior, target)
	{
		var targets = this._targetElements;
		var i = targets ? targets.length : 0;
		while (i-- > 0)
		{
			var registeredTarget = targets[i];
			if (registeredTarget.element == target.element)
			{
				var registeredTargetCount = registeredTarget.targets.length;
				for (var j = 0; j < registeredTargetCount; j++)
				{
					if (registeredTarget.targets[j] == target)
					{
						Array.removeAt(registeredTarget.targets, j);
						break;
					}
				}
				if (registeredTargetCount == 1)
					Array.removeAt(targets, i);
				break;
			}
		}
	},
	_defaultDrop: function(beh)
	{
		var act = beh.get_defaultDropAction();
		var effect = this.get_dragDropEffect();
		if (act <= 0 || effect <= 0)
			return;
		var src = this.get_source().element;
		var target = this.get_target().element;
		if (!target || !src)
			return;
		var elem = src, parent = src.parentNode;
		var move = effect == 1;
		
		if (!move)// || act > 2)
		{
			var div = document.createElement('DIV');
			div.appendChild(elem.cloneNode(true));
			div.innerHTML = div.innerHTML;
			elem = div.firstChild;
		}
		else if (parent == elem)
			return;
		this._defDropElem = elem;
		if (move)
			parent.removeChild(src);
		
		if ((act & 1) == 0)
			target.insertBefore(elem, target.firstChild);
		else
			target.appendChild(elem);
	},
	

	
	__setCursor: function(cursor, clear)
	{
		
		if (this._cursorElem)
		{
			this._cursorElem.style.cursor = cursor;
			return;
		}
		var curElem = this._currentElement, prevElem = this._previousElement;
		if (prevElem)
		{
			if (prevElem.__originalCursor != null)
			{
				prevElem.style.cursor = prevElem.__originalCursor;
				prevElem.__originalCursor = null;
			}
		}
		if (curElem)
		{
			if (clear)
			{
				if (curElem.__originalCursor != null)
				{
					document.body.style.cursor = document.body.__originalCursor;
					curElem.style.cursor = curElem.__originalCursor;
					document.body.__originalCursor = curElem.style.cursor = null;
				}
			}
			else
			{
				if (document.body.__originalCursor == null)
					document.body.__originalCursor = document.body.style.cursor;
				if (curElem.__originalCursor == null)
					curElem.__originalCursor = curElem.style.cursor;
				document.body.style.cursor = curElem.style.cursor = cursor;
			}
		}
	},

	__handleWindowedElems: function(hide)
	{
		var tags = ['IFRAME', 'OBJECT'];
		for (var j = 0; j < 2; j++)
		{
			var elems = document.getElementsByTagName(tags[j]);
			for (var i = 0; i < elems.length; i++)
			{
				var elem = elems[i];
				if (hide)
				{
					var height = elem ? elem.offsetHeight : 0, width = elem ? elem.offsetWidth : 0;
					if (!height || !width || width == 0)
						continue;
					var div = elem.__dragDropDiv = document.createElement('DIV');
					var style = div.style;
					style.height = (height + 4) + 'px';
					style.width = (width + 4) + 'px';
					var pos = Sys.UI.DomElement.getLocation(elem);
					style.top = (pos.y - 2) + 'px';
					style.left = (pos.x - 2) + 'px';
					style.position = 'absolute';
					style.background = 'white';
					style.zIndex = this._zIndex;
					document.body.appendChild(div);
					$util.setOpacity(div, 1);
					continue;
				}
				if (!elem || !elem.__dragDropDiv)
					continue;
				document.body.removeChild(elem.__dragDropDiv);
				elem.__dragDropDiv = null;
			}
		}
	},

	__fireTargetEvent: function(evntName, props)
	{
		var returnVal = false, targets = this._currentTarget;
		if (!targets)
			return false;
		targets = targets.target.targets;
		var i = targets ? targets.length : 0;
		while (i-- > 0)
		{
			var target = targets[i];
			var behavior = target.behavior;
			var dropChannels = behavior._dropChannels;
			if (dropChannels == null || behavior._dropChannels.length == 0)
			{
				returnVal = true;
				if (!behavior._events._fireEvent(evntName, props))
					return false;
			}
			else
			{
				var dragChannels = this._sourceBehavior._dragChannels;
				var j = dragChannels ? dragChannels.length : 0;
				while (j-- > 0)
				{
					if (Array.contains(dropChannels, dragChannels[j]))
					{
						returnVal = true;
						if (!behavior._events._fireEvent(evntName, props))
							return false;
					}
				}
			}
		}
		return returnVal;
	},

	__setupVisibleDragMakup: function(x, y, elem)
	{
		var style, markup = document.createElement("DIV");
		var dragMarkup = this._sourceBehavior.get_dragMarkup();
		if (dragMarkup)
			markup.appendChild(dragMarkup);
		else
		{
			var clone = elem.cloneNode(true);
			style = clone.style;
			
			style.position = 'static';
			style.left = style.top = '0px';
			style.display = '';
			style.visibility = 'visible';
			style.height = elem.offsetHeight + 'px';
			style.width = elem.offsetWidth + 'px';
			markup.appendChild(clone);
		}
		style = markup.style;
		style.position = "absolute";
		style.zIndex = this._zIndex;
		var updateDisplay = elem.style.display == "none";
		if (updateDisplay)
			elem.style.display = "";
		var scrollElem = $util.IsSafari ? document.body : document.documentElement;
		var pos = Sys.UI.DomElement.getLocation(elem);
		if (updateDisplay)
			elem.style.display = "none";
		markup._originalX = x + (this._docMove ? 0 : scrollElem.scrollLeft);
		markup._originalY = y + (this._docMove ? 0 : scrollElem.scrollTop);
		style.top = (markup._originalTop = pos.y) + "px";
		style.left = (markup._originalLeft = pos.x) + "px";
		document.body.appendChild(markup);
		$util.setOpacity(markup, this._sourceBehavior.get_dragMarkupOpacity());
		this._draggingMarkup = markup;
	},

	_showShell: function()
	{
		var style, id = '_ig_DragDropShellFrame';
		var shell = this._dragShell;
		if (!shell)
			this._dragShell = shell = $get(id);
		if (!shell)
		{
			this._dragShell = shell = document.createElement("IFRAME");
			shell.src = 'javascript:new String("<html></html>")';
			shell.frameBorder = 0;
			shell.scrolling = 'no';
			shell.allowtransparency = 'true';
			shell.id = id;
			style = shell.style;
			style.zIndex = this._zIndex + 1;
			style.position = 'absolute';
			style.left = style.top = '0px';
			document.body.appendChild(shell);
			$util.setOpacity(shell, 0);
			if ($util.IsSafari)
				window.setTimeout(Function.createDelegate(this, this._onLoadFrame), 5);
			else
				$addHandler(shell, 'load', this._shellLoadDelegate = Function.createDelegate(this, this._onLoadFrame));
		}
		style = shell.style;
		var rect = $util.getWinRect();
		if (!$util.IsIE)
		{
			var elem = $util.IsSafari ? document.body : document.documentElement;
			
			var sb = $util.IsSafari ? 16 : 18;
			if (rect.height < elem.scrollHeight)
				rect.maxWidth -= sb;
			if (rect.width < elem.scrollWidth)
				rect.maxHeight -= sb;
		}
		style.height = (rect.maxHeight - 2) + 'px';
		style.width = (rect.maxWidth - 1) + 'px';
		style.display = '';
	}
	
}
$IG._DragDropManager.registerClass("Infragistics.Web.UI._DragDropManager");
$IG.DragDropManager = new $IG._DragDropManager();

$IG.DragDropBehavior = function()
{
	///<summary>
	/// An object that allows the developer to specify elements and/or UIObjects that 
	/// should be source elements and/or target elements.  This object handles all aspects 
	/// of a Drag operation and provides hooks such as events that allow the developer to 
	/// interact with the operation. 
	///</summary>
	this.__sources = [];
	this.__targets = [];
	this._events = new $IG.DragDropEvents(this);   
	this._dragMarkupOpacity = 40;
	this._moveCursor = "move";
	this._copyCursor = $util.IsIE ? "default" : "copy";
	this._noneCursor = "not-allowed";
	this._dragDropMode = $IG.DragDropEffects.Default;
	this._defAction = $IG.DragDropAction.None;
	
	this._mouseDownDelegate = Function.createDelegate(this, this._onMouseDown);
	this._clickDelegate = Function.createDelegate(this, this._onClick);
	this._mouseMoveDelegate = Function.createDelegate(this, this._onMouseMove);
	this._dragStartDelegate = Function.createDelegate(this, this._onDragStart);
}

$IG.DragDropBehavior.prototype =
{
	
	_addTarget: function(elem, object, includeChildren)
	{
		var target = { element: elem, object: object, _includeChildren: includeChildren };
		this.__targets.push(target);
		$IG.DragDropManager._registerTarget(this, target);
	},

	_addSource: function(elem, obj)
	{
		var source = { element: elem, object: obj };
		this.__sources.push(source);
		elem.__source = source;
		$addHandler(elem, 'mousedown', this._mouseDownDelegate);
		$addHandler(elem, 'mousemove', this._mouseMoveDelegate);
		$addHandler(elem, 'dragstart', this._dragStartDelegate);
		$addHandler(elem, 'click', this._clickDelegate);
	},

	_removeSrc: function(source)
	{
		var elem = source.element;
		if (!elem || !elem.__source)
			return;
		elem.__source = source.element = null;
		try
		{
			$removeHandler(elem, 'mousedown', this._mouseDownDelegate);
			$removeHandler(elem, 'mousemove', this._mouseMoveDelegate);
			$removeHandler(elem, 'dragstart', this._dragStartDelegate);
			$removeHandler(elem, 'click', this._clickDelegate);
		} catch (ex) { };
	},

	dispose: function()
	{
		///<summary>
		/// Removes event handlers and clears objects.
		///</summary>
		var list = this.__sources;
		var i = list ? list.length : 0;
		while (i-- > 0)
			this._removeSrc(list[i]);
		this.get_events().clearHandlers();
		list = this.__targets;
		i = list ? list.length : 0;
		while (i-- > 0)
		{
			var item = list[i];
			$IG.DragDropManager._unRegisterTarget(this, item);
			item.element = item.behavior = null;
		}
	},
	

	
	_onMouseDown: function(e)
	{
		if (e.button == 0)
		{
			this._mouseDown = true;
			
			if (!$IG.DragDropManager._supportsElemFromPoint || $util.IsFireFox)
				$util.cancelEvent(e);
		}
	},
	_onClick: function(e)
	{
		this._mouseDown = false;
		if ($IG.DragDropManager.get_isDragging())
			$IG.DragDropManager.endDrag(true);
	},
	_onDragStart: function(e)
	{
		$util.cancelEvent(e);
	},
	_onMouseMove: function(e)
	{
		if (!$IG.DragDropManager.get_isDragging())
		{
			if (this._mouseDown)
			{
				var elem = e.target;
				this.__startElem = elem;
				while (elem && !elem.__source)
					elem = elem.parentNode;
				if (elem)
					$IG.DragDropManager._startDrag(this, elem.__source, e);
			}
		}
		this._mouseDown = false;
	},
	

	
	set_currentElementDragMarkup: function(enable)
	{
		///<summary>
		/// Sets option to use html element located under mouse as drag markup, or use whole source element.
		///</summary>
		///<param name="enable" type="Boolean">True: use html element located under mouse. False: use whole source element. Default value is false.</param>
		this._underMouseElem = enable;
	},

	get_currentElementDragMarkup: function()
	{
		///<summary>
		/// Gets an option to use html element located under mouse as drag markup, or use whole source element.
		///</summary>
		/// <returns type="Boolean">Returns true if element located under mouse is used.</returns>
		return this._underMouseElem == true;
	},

	set_dragMarkup: function(markup)
	{
		///<summary>
		/// Sets the custom drag markup that will be used to simulate an element during the 
		/// Drag operation. If no markup is specified, a clone will be made of the source 
		/// element, and that will be used.  
		///</summary>
		///<remarks>
		/// If your drag markup will change based on what is being dragged, you can set the drag 
		/// markup as late as the DragStart event.  If you do not want any drag markup, you can pass 
		/// in a empty element, such as document.createElement(�div�).
		///</remarks>
		///<param name="markup" domElement="true" optional="true" mayBeNull="true">
		/// The html element used as drag markup.
		///</param>
		this._dragMarkup = markup;
	},

	get_dragMarkup: function()
	{
		///<summary>
		/// Gets the custome drag markup that will be used to simulate an element during the 
		/// Drag operation.
		///</summary>
		/// <returns type="Object" mayBeNull="true">Reference to html element which was set by set_dragMarkup.</returns>
		return this._dragMarkup ? this._dragMarkup : null;
	},

	set_dragMarkupOpacity: function(opacity)
	{
		///<summary>
		/// Sets the opacity of the drag markup. 
		///</summary>
		///<remarks>
		/// The method expects a value between 0 and 100, where 0 is invisible and 100 is opaque.  
		/// By default the opacity is set to 40.
		///</remarks>
		///<param name="opacity" type="Number" optional="false" mayBeNull="false">
		/// Opacity for drag markup element.
		///</param>
		this._dragMarkupOpacity = opacity;
	},

	get_dragMarkupOpacity: function()
	{
		///<summary>
		/// Gets/sets the opacity of the drag markup. 
		///</summary>
		/// <returns type="Number">Opacity of drag markup element.</returns>
		return this._dragMarkupOpacity;
	},

	set_moveCursor: function(cursor, isUrl)
	{
		///<summary>
		/// Sets the cursor that should be used if the DragDropEffect is Move.
		///</summary>
		///<param name="cursor" type="String">
		/// Either a default browser cursor name, or a custom url. 
		///</param>
		///<param name="isUrl" type="Boolean" optional="true" mayBeNull="true">
		/// Specifies whether the cursor is a default browser cursor (false), or a custom url(true). 
		///</param>
		this._moveCursor = isUrl ? ("url(" + cursor + "), move") : cursor;
	},

	set_copyCursor: function(cursor, isUrl)
	{
		///<summary>
		/// Sets the cursor that should be used if the DragDropEffect is Copy.
		///</summary>
		///<param name="cursor" type="String">
		/// Either a default browser cursor name, or a custom url. 
		///</param>
		///<param name="isUrl" type="Boolean" optional="true" mayBeNull="true">
		/// Specifies whether the cursor is a default browser cursor (false), or a custom url(true). 
		///</param>
		this._copyCursor = isUrl ? ("url(" + cursor + "), copy") : cursor;
	},

	set_noneCursor: function(cursor, isUrl)
	{
		///<summary>
		/// Sets the cursor that should be used if the DragDropEffect is None.
		///</summary>
		///<param name="cursor" type="String">
		/// Either a default browser cursor name, or a custom url. 
		///</param>
		///<param name="isUrl" type="Boolean"  optional="true" mayBeNull="true">
		/// Specifies whether the cursor is a default browser cursor (false), or a custom url(true). 
		///</param>
		this._noneCursor = isUrl ? ("url(" + cursor + "), not-allowed") : cursor;
	},

	set_dragDropMode: function(effect)
	{
		///<summary>
		/// If set, the DragDropEffect won�t be overridable via the keyboard. 
		/// Which means it doesn�t matter if the user presses the Ctrl or Shift key, 
		/// because the specified effect will remain.  
		///</summary>
		///<param name="effect" type="Infragistics.Web.UI.DragDropEffects">
		/// The DragDropEffects to override. 
		///</param>
		///<remarks>
		/// To turn off this behavior the developer can pass in a DragDropEffect of Default. 
		///</remarks>
		this._dragDropMode = effect;
	},

	set_defaultDropAction: function(action)
	{
		///<summary>
		/// Sets default action which should occur when source element was dropped on target.
		///</summary>
		///<param name="action" type="Infragistics.Web.UI.DragDropAction">
		/// The default action which occurs on drop event.
		///</param>
		///<remarks>
		/// The value of None means that application should implement drop action.
		/// Otherwise, action will depend on get_dragDropEffect() of DragDropManager.
		/// If dragDropEffect is Copy and value is set to Append, then
		/// the copy/clone of source element will be appended to target.
		/// If dragDropEffect is Copy and value is set to Insert, then
		/// the copy/clone of source element will be inserted in the target as first child.
		/// If dragDropEffect is Move and value is set to Append, then source
		/// element will be removed from it original location and appended to target.
		/// If dragDropEffect is Move and value is set to Insert, then source
		/// element will be removed from it original location and inserted into target as first child.
		/// So, source element can be dragged and dropped on another target.
		///</remarks>
		this._defAction = action;
	},

	get_defaultDropAction: function()
	{
		///<summary>Gets the default drop action</summary>
		/// <returns type="Infragistics.Web.UI.DragDropAction">Default action on drop event.</returns>
		return this._defAction;
	},

	addTargetElement: function(element, includeChildren)
	{
		///<summary>
		/// Notifies the behavior that the following element is a possible drop target, 
		/// which means that events such as DragEnter, DragLeave, DragMove, and Drop will 
		/// fire when the mouse enters the element while in a Drag operation.
		///</summary>
		///<param name="element" domElement="true">
		/// The element that should be marked as droppable.
		///</param>
		///<param name="includeChildren" type="Boolean" optional="true" mayBeNull="true">
		/// Determines whether the DragEnter and DragLeave events should fire only when 
		/// the mouse enters the specified target element (false), or for each child element 
		/// of the specified target element (true).
		///</param>
		this._addTarget(element, null, includeChildren);
	},

	addTargetObject: function(obj, includeChildren)
	{
		///<summary>
		/// Notifies the behavior that the following element associated with the object is 
		/// a possible drop target, which means that events such as DragEnter, DragLeave, 
		/// DragMove, and Drop will fire when the mouse enters the element while in a Drag operation.  
		///</summary>
		///<param name="obj" type="Infragistics.Web.UI.UIObject">
		/// The object whose associated element should be marked droppable.
		///</param>
		///<param name="includeChildren" type="Boolean" optional="true" mayBeNull="true">
		/// Determines whether the DragEnter and DragLeave events should fire only when 
		/// the mouse enters the specified target element (false), or for each child element 
		/// of the specified target element (true).
		///</param>
		this._addTarget(obj.get_element(), obj, includeChildren);
	},

	removeTarget: function(obj)
	{
		///<summary>
		/// Removes target which was set by addTargetElement or by addTargetObject.
		///</summary>
		///<param name="obj">
		/// Reference to DOM element or to Infragistics.Web.UI.UIObject.
		///</param>
		var elem = obj;
		if (!obj)
			return;
		if (typeof obj.get_element == 'function')
		{
			elem = obj.get_element();
			if (!elem)
				return;
		}
		else
			obj = null;
		var list = this.__targets;
		var i = list ? list.length : 0;
		while (i-- > 0)
		{
			var item = list[i];
			if (!item || item.element != elem || item.object != obj)
				continue;
			$IG.DragDropManager._unRegisterTarget(this, item);
			item.element = item.behavior = null;
			Array.removeAt(list, i);
		}
	},

	addSourceElement: function(elem)
	{
		///<summary>
		/// Notifies the behavior that the following element is draggable. 
		///</summary>
		///<param name="elem" domElement="true">
		/// The element that should be marked draggable.
		///</param>
		this._addSource(elem, null);
	},

	addSourceObject: function(obj)
	{
		///<summary>
		/// Notifies the behavior that the element associated with the object is draggable.
		///</summary>
		///<param name="obj" type="Infragistics.Web.UI.UIObject">
		/// The object whose associated element should be marked draggable.
		///</param>
		this._addSource(obj.get_element(), obj);
	},

	removeSource: function(obj)
	{
		///<summary>
		/// Removes source which was set by addSourceElement or by addSourceObject.
		///</summary>
		///<param name="obj">
		/// Reference to DOM element or to Infragistics.Web.UI.UIObject.
		///</param>
		var elem = obj;
		if (!obj)
			return;
		if (typeof obj.get_element == 'function')
		{
			elem = obj.get_element();
			if (!elem)
				return;
		}
		else
			obj = null;
		var list = this.__sources;
		var i = list ? list.length : 0;
		while (i-- > 0)
		{
			var item = list[i];
			if (!item || item.element != elem || item.object != obj)
				continue;
			this._removeSrc(item);
			Array.removeAt(list, i);
		}
	},

	addDragChannels: function(channels)
	{
		///<summary>
		/// Sets the channels that the source object is allowed to be dragged onto.
		///</summary>
		///<param name="channels" type="Array">
		/// An array of keys (strings or integers).
		///</param>
		///<remarks>
		/// Once a channel is specified, a source object will only be allowed to be dragged 
		/// onto a target that is listening on the same channel, or a target that is not listening 
		/// on any channels.  
		/// One use of Channels is to protect a Infragistics.Web.UI.DragDropBehavior from receiving 
		/// unwanted drop elements from another DragDropBehavior.
		///</remarks>
		if (!this._dragChannels)
			this._dragChannels = [];
		Array.addRange(this._dragChannels, channels);
	},

	addDropChannels: function(channels)
	{
		///<summary>
		/// Sets the channels that a target object is listening for.
		///</summary>
		///<param name="channels" type="Array">
		/// An array of keys (strings or integers).
		///</param>
		///<remarks>
		/// Once a channel is specified, a target object will only allow source objects that are on 
		/// the same channel.
		/// One use of Channels is to protect a Infragistics.Web.UI.DragDropBehavior from receiving 
		/// unwanted drop elements from another DragDropBehavior.
		///</remarks>
		if (!this._dropChannels)
			this._dropChannels = [];
		Array.addRange(this._dropChannels, channels);
	},

	get_events: function()
	{
		///<summary>
		/// Returns a reference to the Infragistics.Web.UI.DragDropEvents object, which contains all the events of the behavior.
		///</summary>
		/// <returns type="Infragistics.Web.UI.DragDropEvents">Reference to events.</returns>
		return this._events;
	}
	
};
$IG.DragDropBehavior.registerClass("Infragistics.Web.UI.DragDropBehavior");

$IG.DragDropEvents = function (behavior) 
{
    ///<summary>
    /// An object that allows the developer to attach event listeners associated with a 
    /// Infragistics.Web.UI.DragDropBehavior. All handlers should have the following signature: 
    /// handler (behavior, evntArgs)
    ///</summary>
    this._handlers ={};
    this._behavior = behavior;
}
$IG.DragDropEvents.prototype =
{
	addDragStartHandler: function(handler)
	{
		///<summary>
		/// Attaches an event listener to the DragStart event of a Infragistics.Web.UI.DragDropBehavior.          
		///</summary>
		///<param name="handler" type="Function">
		/// The function that should be called when the event is fired.
		///</param>
		///<remarks>
		/// Fired when a drag operation is about to begin.  This event is cancelable. 
		/// The EventArgs are of type: Infragistics.Web.UI.DragDropCancelEventArgs.
		///</remarks>
		this.__addHandler("DragStart", handler, $IG.DragDropCancelableMoveEventArgs);
		//this.__addHandler("DragStart", handler, $IG.DragDropCancelEventArgs);
	},

	addDropHandler: function(handler)
	{
		///<summary>
		/// Attaches an event listener to the Drop event of a Infragistics.Web.UI.DragDropBehavior.          
		///</summary>
		///<param name="handler" type="Function">
		/// The function that should be called when the event is fired.
		///</param>
		///<remarks>
		/// Fired when a dragged element is released over a target element.  
		/// The EventArgs are of type: Infragistics.Web.UI.DragDropEventArgs. 
		///</remarks>
		this.__addHandler("Drop", handler, $IG.DragDropEventArgs);
	},

	addDragEnterHandler: function(handler)
	{
		///<summary>
		/// Attaches an event listener to the DragEnter event of a Infragistics.Web.UI.DragDropBehavior.          
		///</summary>
		///<param name="handler" type="Function">
		/// The function that should be called when the event is fired.
		///</param>
		///<remarks>
		/// Fired when a dragged element enters a target element.  This event is cancelable.   
		/// The EventArgs are of type: Infragistics.Web.UI.DragDropCancelEventArgs. 
		///</remarks>
		this.__addHandler("DragEnter", handler, $IG.DragDropCancelEventArgs);
	},

	addDragLeaveHandler: function(handler)
	{
		///<summary>
		/// Attaches an event listener to the DragLeave event of a Infragistics.Web.UI.DragDropBehavior.          
		///</summary>
		///<param name="handler" type="Function">
		/// The function that should be called when the event is fired.
		///</param>
		///<remarks>
		/// Fired when a dragged element leaves a target element.   
		/// The EventArgs are of type: Infragistics.Web.UI.DragDropEventArgs. 
		///</remarks>
		this.__addHandler("DragLeave", handler, $IG.DragDropEventArgs);
	},

	addDragCancelHandler: function(handler)
	{
		///<summary>
		/// Attaches an event listener to the DragCancel event of a Infragistics.Web.UI.DragDropBehavior.          
		///</summary>
		///<param name="handler" type="Function">
		/// The function that should be called when the event is fired.
		///</param>
		///<remarks>
		/// Fired when a drag operation is canceled. This can happen if the dragged element is 
		/// released while not over a target element, the escape key is pressed, or a developer 
		/// calls the endDrag method off of the DragDropManager with a parameter of true.  
		/// The EventArgs are of type: Infragistics.Web.UI.DragDropEventArgs. 
		///</remarks>
		this.__addHandler("DragCancel", handler, $IG.DragDropEventArgs);
	},

	addDragMoveHandler: function(handler)
	{
		///<summary>
		/// Attaches an event listener to the DragMove event of a Infragistics.Web.UI.DragDropBehavior.          
		///</summary>
		///<param name="handler" type="Function">
		/// The function that should be called when the event is fired.
		///</param>
		///<remarks>
		/// Fired every time the mouse moves while over a target element.   
		/// The EventArgs are of type: Infragistics.Web.UI.DragDropMoveEventArgs. 
		///</remarks>
		this.__addHandler("DragMove", handler, $IG.DragDropMoveEventArgs);
	},

	addDragEndHandler: function(handler)
	{
		///<summary>
		/// Attaches an event listener to the DragEnd event of a Infragistics.Web.UI.DragDropBehavior.          
		///</summary>
		///<param name="handler" type="Function">
		/// The function that should be called when the event is fired.
		///</param>
		///<remarks>
		/// Fired every time a drag operation ends, whether it was canceled or a drop occurred. 
		/// This event is always the last event fired.   
		/// The EventArgs are of type: Infragistics.Web.UI.DragDropEventArgs. 
		///</remarks>
		this.__addHandler("DragEnd", handler, $IG.DragDropEventArgs);
	},

	removeDragStartHandler: function(handler)
	{
		///<summary>
		///Removes the event listener to the DragStart event of a Infragistics.Web.UI.DragDropBehavior.          
		///</summary>
		///<param name="handler" type="Function">
		/// The function that was used originally to fire the event..
		///</param>
		this.__removeHandler("DragStart", handler);
	},

	removeDropHandler: function(handler)
	{
		///<summary>
		///Removes the event listener to the Drop event of a Infragistics.Web.UI.DragDropBehavior.          
		///</summary>
		///<param name="handler" type="Function">
		/// The function that was used originally to fire the event..
		///</param>
		this.__removeHandler("Drop", handler);
	},

	removeDragEnterHandler: function(handler)
	{
		///<summary>
		///Removes the event listener to the DragEnter event of a Infragistics.Web.UI.DragDropBehavior.          
		///</summary>
		///<param name="handler" type="Function">
		/// The function that was used originally to fire the event..
		///</param>
		this.__removeHandler("DragEnter", handler);
	},

	removeDragLeaveHandler: function(handler)
	{
		///<summary>
		///Removes the event listener to the DragLeave event of a Infragistics.Web.UI.DragDropBehavior.          
		///</summary>
		///<param name="handler" type="Function">
		/// The function that was used originally to fire the event..
		///</param>
		this.__removeHandler("DragLeave", handler);
	},

	removeDragCancelHandler: function(handler)
	{
		///<summary>
		///Removes the event listener to the DragCancel event of a Infragistics.Web.UI.DragDropBehavior.          
		///</summary>
		///<param name="handler" type="Function">
		/// The function that was used originally to fire the event..
		///</param>
		this.__removeHandler("DragCancel", handler);
	},

	removeDragMoveHandler: function(handler)
	{
		///<summary>
		///Removes the event listener to the DragMove event of a Infragistics.Web.UI.DragDropBehavior.          
		///</summary>
		///<param name="handler" type="Function">
		/// The function that was used originally to fire the event..
		///</param>
		this.__removeHandler("DragMove", handler);
	},

	removeDragEndHandler: function(handler)
	{
		///<summary>
		///Removes the event listener to the DragEnd event of a Infragistics.Web.UI.DragDropBehavior.          
		///</summary>
		///<param name="handler" type="Function">
		/// The function that was used originally to fire the event..
		///</param>
		this.__removeHandler("DragEnd", handler);
	},

	clearHandlers: function()
	{
		this._handlers = {};
	},

	_removeHandler: function(name, handler)
	{
		var handlers = this._handlers[name];
		var i = handlers ? handlers.length : 0;
		while (i-- > 0)
		{
			if (handlers[i][0] == handler)
			{
				Array.removeAt(handlers, i);
				break;
			}
		}
	},

	__addHandler: function(name, handler, args)
	{
		if (!this._handlers[name])
			this._handlers[name] = [];
		this._handlers[name].push([handler, args]);
	},

	_fireEvent: function(name, evntArgs)
	{
		var handlers = this._handlers[name];
		var i = handlers ? handlers.length : 0;
		while (i-- > 0)
		{
			var handler = handlers[i];
			var evnt = handler[0];
			var args = new handler[1](evntArgs);
			evnt(this._behavior, args);
			if (args._cancel)
				return false;
		}
		return true;
	}
}
$IG.DragDropEvents.registerClass("Infragistics.Web.UI.DragDropEvents");

$IG.DragDropEventArgs = function() 
{
	///<summary>
	/// Provides data for Drag and Drop events of a DragDropBehavior.
	///</summary>
}
$IG.DragDropEventArgs.prototype = 
{
	get_manager:function()
	{
		///<summary>
		/// Returns a references to the DragDropManager object.
		///</summary>
		/// <returns type="Infragistics.Web.UI._DragDropManager">Reference to DragDropManager.</returns>
		return $IG.DragDropManager;
	}
}
$IG.DragDropEventArgs.registerClass("Infragistics.Web.UI.DragDropEventArgs");

$IG.DragDropMoveEventArgs = function (point) 
{
	///<summary>
	/// Inherits from DragDropEventArgs and adds additional information such as x and y coordinates.
	///</summary> 
	///<param name="point" type="Object">Object that contains x and y members.</param>
	this._point = point;
	$IG.DragDropMoveEventArgs.initializeBase(this);
}
$IG.DragDropMoveEventArgs.prototype = 
{
	get_x:function()
	{
		///<summary>
		/// Returns the current x coordinate of the mouse in relation to the window. 
		///</summary>
		/// <returns type="Number">The current x coordinate.</returns>
		return this._point.x;
	},
	get_y:function()
	{
		///<summary>
		/// Returns the current y coordinate of the mouse in relation to the window.  
		///</summary>
		/// <returns type="Number">The current y coordinate.</returns>
		return this._point.y;
	}
}
$IG.DragDropMoveEventArgs.registerClass("Infragistics.Web.UI.DragDropMoveEventArgs", $IG.DragDropEventArgs);

$IG.DragDropCancelEventArgs = function () 
{
	///<summary>
	/// Inherits from DragDropEventArgs , but is used for events that are cancelable. 
	///</summary>
	this._cancel = false;
	$IG.DragDropCancelEventArgs.initializeBase(this);
}
$IG.DragDropCancelEventArgs.prototype = 
{
	get_cancel:function()
	{
		///<summary>
		/// Checks whether the event should be canceled. 
		///</summary>
		/// <returns type="Boolean">True if event was canceled.</returns>
		return this._cancel;
	},
	set_cancel:function(cancel)
	{
		///<summary>
		/// Sets whether the event should be canceled. 
		///</summary>
		///<param name="cancel" type="Boolean">Use true to cancel event.</param>
		this._cancel = cancel;
	}
}
$IG.DragDropCancelEventArgs.registerClass("Infragistics.Web.UI.DragDropCancelEventArgs", $IG.DragDropEventArgs);

$IG.DragDropCancelableMoveEventArgs = function (point) 
{
	///<summary>
	/// Inherits from DragDropMoveEventArgs , but is used for events that are cancelable. 
	///</summary>
	///<param name="point" type="Object">Object that contains x and y members.</param>
	this._cancel = false;
	$IG.DragDropCancelableMoveEventArgs.initializeBase(this, [point]);
}
$IG.DragDropCancelableMoveEventArgs.prototype = 
{
	get_cancel:function()
	{
		///<summary>
		/// Checks whether the event should be canceled. 
		///</summary>
		/// <returns type="Boolean">True if event was canceled.</returns>
		return this._cancel;
	},
	set_cancel:function(cancel)
	{
		///<summary>
		/// Sets whether the event should be canceled. 
		///</summary>
		///<param name="cancel" type="Boolean">Use true to cancel event.</param>
		this._cancel = cancel;
	}
}
$IG.DragDropCancelableMoveEventArgs.registerClass("Infragistics.Web.UI.DragDropCancelableMoveEventArgs", $IG.DragDropMoveEventArgs);

